/*
Author - lukasz1985
Script based upon MinimalGS library: 
http://www.tt-forums.net/viewtopic.php?f=65&t=62163

The license:
*/

/*
 * This file is part of MinimalGS, which is a GameScript for OpenTTD
 * Copyright (C) 2012-2013  Leif Linse
 *
 * MinimalGS is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License
 *
 * MinimalGS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MinimalGS; If not, see <http://www.gnu.org/licenses/> or
 * write to the Free Software Foundation, Inc., 51 Franklin Street, 
 * Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

// Namespace declaration
eco <- {};

// Temporary solution
const COMPANY_SELF = 0;

require("cargo.nut");
require("economy.nut");
require("towns_manager.nut");
require("industries_manager.nut");
require("impacted.nut");

require("impactor.nut");
require("growth_impactor.nut");

require("tile.nut");
require("sign.nut")
require("town.nut");
require("industry.nut");
require("tile.nut");

require("manager.nut");
require("vehicle.nut");


class Main extends GSController 
{
	days_passed = 0;
	economy = null;
	manager = null;
	current_date = null;
	_load_data = null;
	
	constructor(){}
	
	function Start(){
		Init();
		current_date = GSDate.GetCurrentDate();
		
		// Wait for the game to start
		this.Sleep(1);
	
		// Welcome human player
	
		InitSettings();
		
		economy = eco.Economy();
		economy.CleanUp();
		economy.Initialize();
		manager = eco.Manager();
		
		
		
		/**
		The loop 
		*/
		while (true) {
			local loop_start_tick = GSController.GetTick();
			local current_date = GSDate.GetCurrentDate();
			local days_since_last_update = current_date - this.current_date;
			this.current_date = current_date; 
			if (days_since_last_update >= 1) {
				days_passed += days_since_last_update;
				Update();
			}
	
			// Loop with a frequency of five days
			local ticks_used = GSController.GetTick() - loop_start_tick;
			
			Sleep(max(1, 1 * 74 - ticks_used));
		}
		
		while(GSEventController.IsEventWaiting()) {
			HandleEvents();
		}
	}
	
	/**
	Events handling for industries and vehicle crashes
	*/
	function HandleEvents() {
		if(GSEventController.IsEventWaiting())	{
			local event = GSEventController.GetNextEvent();
			economy.HandleEvent(event);
			manager.HandleEvent(event);
		}
	}
	
	function Update() {
		local current_date = GSDate.GetCurrentDate();
		local day_of_month = GSDate.GetDayOfMonth(current_date)
		if (day_of_month  == 1 ){
			// Every first of month
			
			MonthlyUpdate();
		}
		
		// Every day
		manager.DailyUpdate();
		
		if(GSEventController.IsEventWaiting())	{
			local event = GSEventController.GetNextEvent();
			economy.HandleEvent(event);
			manager.HandleEvent(event);
		}
	}
	
	function MonthlyUpdate() {
		manager.MonthlyUpdate();
		economy.Update();
	}
	
	
	function Init()
	{
		if (this._load_data != null)
		{
			// Copy loaded data from this._load_data to this.*
			// or do whatever with the loaded data
		}
	}
	
	function Save()
	{
		return { 
			some_data = null,
			some_other_data = null
		};
	}
	
	function Load(version, tbl)
	{
	
		// Store a copy of the table from the save game
		// but do not process the loaded data yet. Wait with that to Init
		// so that OpenTTD doesn't kick us for taking too long to load.
		this._load_data = {}
	   	foreach(key, val in tbl)
		{
			this._load_data.rawset(key, val);
		}	
	}
	
	function InitSettings()
	{
		GSGameSettings.SetValue("economy.town_growth_rate", 4);
	}
}

